/*
title:		uvw.ms
path:		$scripts\AShimScripts\functions
autor:		AShim (Titov Mykhaylo)
e-mail:		ashim@rambler.ru
date:		february 2010
version:	1.0
*/

global AShim_uvw
if (AShim_uvw != true) do
(
	include "$scripts/AShimScripts/functions/common.ms"
	
	global AShim_getUVWModifier	
	fn AShim_getUVWModifier obj = (
		if (classof obj != PolyMeshObject) do
			return undefined
		UVW = obj.modifiers[1]
		if (classof UVW != Unwrap_UVW) do
			return undefined
		UVW
	)
	
	global AShim_getUVWElementsSelectionFromFaces
	fn AShim_getUVWElementsSelectionFromFaces obj = (
		if ((UVW = AShim_getUVWmodifier obj) == undefined) do return undefined
		selected_faces = UVW.getSelectedFacesByNode obj
		store_selected_faces = selected_faces
		elements = #()
		selected_faces_in_element = #()
		while (NOT selected_faces.isEmpty) do (
			UVW.selectFacesByNode #{(selected_faces as array)[1]} obj
			UVW.selectElement()
			faces_in_element = UVW.getSelectedFacesByNode obj
			append elements faces_in_element
			append selected_faces_in_element (faces_in_element * store_selected_faces)
			selected_faces -= faces_in_element
		)
		UVW.selectFacesByNode store_selected_faces obj
		len = elements.count
		if (len > 0) then (
			for i = 1 to elements.count collect #(selected_faces_in_element[i], elements[i])
		)
		else (
			undefined
		)
	)
	
	global AShim_getCanBeBorder
	fn AShim_getCanBeBorder border_id set_id uvw_vertices_pos = (
		local len = border_id.count - 1
		local i, ABC, ABC_perpendicular, bound
		local can_be_border = (for i = 1 to len collect #{})
		ABC = (for i = 1 to len collect (AShim_getABCLine uvw_vertices_pos[border_id[i]] uvw_vertices_pos[border_id[i + 1]]))
		ABC_perpendicular = (for i = 1 to len collect (AShim_getABCPerpendicular ABC[i]))
		bound = (for i = 1 to len collect #(AShim_getDistToABCLine ABC_perpendicular[i] uvw_vertices_pos[border_id[i]], AShim_getDistToABCLine ABC_perpendicular[i] uvw_vertices_pos[border_id[i+1]]))
		for id in set_id do (
			for i = 1 to len do (
				if ((id != border_id[i]) AND (id != border_id[i+1]) \
					AND ((AShim_getDistToABCLine ABC[i] uvw_vertices_pos[id]) < 0) \
					AND ((AShim_getDistToABCLine ABC_perpendicular[i] uvw_vertices_pos[id]) < bound[i][2]) \
					AND ((AShim_getDistToABCLine ABC_perpendicular[i] uvw_vertices_pos[id]) > bound[i][1]) \
					)  do (
					append can_be_border[i] id
				)
			)
		)
		#(can_be_border, ABC)
	)
	
	global AShim_getBorderRefine
	fn AShim_getBorderRefine border_id can_be_border ABC uvw_vertices_pos = (
		between_id = undefined
		between_dist = inf
		for id in can_be_border do (
			dist = AShim_getDistToABCLine ABC uvw_vertices_pos[id]
			if (dist < between_dist) do (
				between_id = id
				between_dist = dist
			)
		)
		if (between_dist < 0) then (
			can_be_border_between = AShim_getCanBeBorder #(border_id[1], between_id, border_id[2]) (can_be_border - #{border_id[1], between_id, border_id[2]}) uvw_vertices_pos
			(AShim_getBorderRefine #(border_id[1], between_id) can_be_border_between[1][1] can_be_border_between[2][1] uvw_vertices_pos) + \
			#(between_id) + \
			(AShim_getBorderRefine #(between_id, border_id[2]) can_be_border_between[1][2] can_be_border_between[2][2] uvw_vertices_pos)
		)
		else (
			#()
		)
	)
	
	global AShim_getUVWBorder
	fn AShim_getUVWBorder obj uvw_vertices = (
		if ((UVW = AShim_getUVWmodifier obj) == undefined) do return undefined
		uvw_vertices = try (uvw_vertices as array) catch undefined
		if (uvw_vertices == undefined) do return undefined
		if (uvw_vertices.count < 2) do return uvw_vertices
		uvw_vertices_pos = (for vert in uvw_vertices collect (UVW.getVertexPositionByNode 0 vert obj))
		max_min_values = (for i = 1 to 8 collect inf);
		border_draft = (for i = 1 to 8 collect undefined);
		mul = #(#(-1 , 0), #(-1, 1), #(0, 1), #(1, 1), #(1, 0), #(1, -1), #(0, -1), #(-1, -1))
		for i = 1 to uvw_vertices.count do (
			for j = 1 to 8 do (
				if ((uvw_vertices_pos[i].x * mul[j][1] + uvw_vertices_pos[i].y * mul[j][2]) < max_min_values[j]) do (
					max_min_values[j] = uvw_vertices_pos[i].x * mul[j][1] + uvw_vertices_pos[i].y * mul[j][2]
					border_draft[j] = i
				)
			)
		)
		d_max = (amax (-max_min_values[1] - max_min_values[5]) (-max_min_values[3] - max_min_values[7])) / 10000
		border_draft = makeUniqueArray border_draft
		len = border_draft.count
		border_draft += #(border_draft[1])
		can_be_border = AShim_getCanBeBorder border_draft (#{1..uvw_vertices.count} - (border_draft as BitArray)) uvw_vertices_pos
		border_refine = #()
		for i = 1 to len do (
			border_refine += #(border_draft[i]) + \
			(AShim_getBorderRefine #(border_draft[i], border_draft[i+1]) can_be_border[1][i] can_be_border[2][i] uvw_vertices_pos)
		)
		return (for v in border_refine collect uvw_vertices[v])
	)
	
	global AShim_findMinAreaDirection
	fn AShim_findMinAreaDirection obj uvw_border = (
		if (classof uvw_border != Array) do return undefined
		if ((UVW = AShim_getUVWmodifier obj) == undefined) do return undefined
		len = uvw_border.count
		if (len < 2) do return undefined
		uvw_border += #(uvw_border[1])
		uvw_border_pos = (for vert in uvw_border collect (UVW.getVertexPositionByNode 0 vert obj))
		ABC = (for i = 1 to len collect (AShim_getABCLine uvw_border_pos[i] uvw_border_pos[i+1]))
		ABC_perpendicular = (for i = 1 to len collect (AShim_getABCPerpendicular ABC[i]))
		bounding_box_width = (for i = 1 to len collect -inf)
		bounding_box_height_min = (for i = 1 to len collect inf)
		bounding_box_height_max = (for i = 1 to len collect -inf)
		for i = 1 to len do (
			for j = 1 to len do (
				width = AShim_getDistToABCLine ABC[i] uvw_border_pos[j]
				height = AShim_getDistToABCLine ABC_perpendicular[i] uvw_border_pos[j]
				if (width > bounding_box_width[i]) do (
					bounding_box_width[i] = width
				)
				if (height > bounding_box_height_max[i]) do (
					bounding_box_height_max[i] = height
				)
				if (height < bounding_box_height_min[i]) do (
					bounding_box_height_min[i] = height
				)
			)
		)
		bounding_box_height = (for i = 1 to len collect (bounding_box_height_max[i] - bounding_box_height_min[i]))
		ABC_area = (for i = 1 to len collect (bounding_box_height[i] * bounding_box_width[i]))
		id = 1
		for i = 2 to len do (
			if (ABC_area[i] <= ABC_area[id]) do (
				id = i
			)
		)
		if (bounding_box_height[id] < bounding_box_width[id]) then (
			#(ABC[id], ABC_area[id])
		)
		else (
			#(ABC_perpendicular[id], ABC_area[id])
		)
	)
	
	print "AShimScripts: uvw.ms has been loaded."
	AShim_uvw = true
)
